﻿using System;
using System.Globalization;
using System.IO;

namespace nam2line_load_csv
{
	class Program
	{
	    private const int INPUT_BUFFER_SIZE = 8 * 1024^2;
        private const int OUTPUT_BUFFER_SIZE = 8 * 1024;


		static void Main(string[] args)
		{
			if (args.Length != 4)
			{
                Console.WriteLine("Trace.nam To Line load in CSV format");
                Console.WriteLine("Author: Martin Sloup, arcao@arcao.com");
				Console.WriteLine("Arguments format: <trace.nam> <output_file> <node_from> <node_to>");
				return;
			}

			string inputFileName = args[0];
			string outputFileName = args[1];
			int fromID = int.Parse(args[2]);
			int toID = int.Parse(args[3]);

			int nextCheckTime = 0;
			int receivedBytes = 0;
			int transmitedBytes = 0;

		    int lineCounter = 0;

            try
            {

                using (StreamReader reader = new StreamReader(new FileStream(inputFileName,FileMode.Open, FileAccess.Read, FileShare.ReadWrite, INPUT_BUFFER_SIZE)))
                {
                    using (StreamWriter writer = new StreamWriter(new FileStream(outputFileName,FileMode.Create, FileAccess.Write, FileShare.Read, OUTPUT_BUFFER_SIZE)))
                    {
                        writer.WriteLine(string.Format(CultureInfo.InvariantCulture, "Time;{0} -> {1};{1} -> {0}", fromID, toID));
                        String line;
                        while ((line = reader.ReadLine()) != null)
                        {
                            lineCounter++;
                            if (line.Length == 0 || (line[0] != 'h' && line[0] != 'r' && line[0] != 'd'))
                                continue;
                            string[] items = line.Split(new char[] {' '});
                            float time = float.Parse(items[2], CultureInfo.InvariantCulture);

                            if (line[0] != 'h')
                                continue;

                            if (time > nextCheckTime + 1)
                            {
                                while (time - nextCheckTime > 2)
                                {
                                    nextCheckTime++;
                                    writer.WriteLine(string.Format(CultureInfo.InvariantCulture, "{0};{1};{2}", nextCheckTime, 0, 0));
                                }
                                nextCheckTime++;

                                writer.WriteLine(string.Format(CultureInfo.InvariantCulture, "{0};{1};{2}", nextCheckTime, transmitedBytes,
                                                               receivedBytes));

                                receivedBytes = 0;
                                transmitedBytes = 0;
                            }

                            int nodeFrom = int.Parse(items[4], CultureInfo.InvariantCulture);
                            int nodeTo = int.Parse(items[6], CultureInfo.InvariantCulture);

                            int packetSize = int.Parse(items[8], CultureInfo.InvariantCulture);

                            if (nodeFrom == fromID && nodeTo == toID)
                                transmitedBytes += packetSize;
                            if (nodeFrom == toID && nodeTo == fromID)
                                receivedBytes += packetSize;

                        }

                        if (receivedBytes > 0 || transmitedBytes > 0)
                        {
                            nextCheckTime++;
                            writer.WriteLine(string.Format(CultureInfo.InvariantCulture, "{0};{1};{2}", nextCheckTime, transmitedBytes, receivedBytes));
                        }
                    }
                }
            } 
            catch(IOException e)
            {
                Console.WriteLine("Error with file: {0}", e.Message);
            }
            catch(FormatException e)
            {
                Console.WriteLine("Error on line {0} in {1}: {2}", lineCounter, inputFileName, e.Message);
            }
		}
	}
}
